package org.eclipse.spliteditor;

import org.eclipse.jface.text.IAutoEditStrategy;
import org.eclipse.jface.text.IAutoIndentStrategy;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IEditingSupport;
import org.eclipse.jface.text.IEditingSupportRegistry;
import org.eclipse.jface.text.IEventConsumer;
import org.eclipse.jface.text.IFindReplaceTarget;
import org.eclipse.jface.text.IFindReplaceTargetExtension;
import org.eclipse.jface.text.IFindReplaceTargetExtension3;
import org.eclipse.jface.text.IPainter;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.IRewriteTarget;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.ITextInputListener;
import org.eclipse.jface.text.ITextListener;
import org.eclipse.jface.text.ITextOperationTarget;
import org.eclipse.jface.text.ITextOperationTargetExtension;
import org.eclipse.jface.text.ITextPresentationListener;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.ITextViewerExtension;
import org.eclipse.jface.text.ITextViewerExtension2;
import org.eclipse.jface.text.ITextViewerExtension3;
import org.eclipse.jface.text.ITextViewerExtension4;
import org.eclipse.jface.text.ITextViewerExtension5;
import org.eclipse.jface.text.ITextViewerExtension6;
import org.eclipse.jface.text.ITextViewerExtension7;
import org.eclipse.jface.text.ITextViewerExtension8;
import org.eclipse.jface.text.IUndoManager;
import org.eclipse.jface.text.IViewportListener;
import org.eclipse.jface.text.IWidgetTokenKeeper;
import org.eclipse.jface.text.IWidgetTokenOwner;
import org.eclipse.jface.text.IWidgetTokenOwnerExtension;
import org.eclipse.jface.text.TextPresentation;
import org.eclipse.jface.text.hyperlink.IHyperlinkDetector;
import org.eclipse.jface.text.quickassist.IQuickAssistAssistant;
import org.eclipse.jface.text.quickassist.IQuickAssistInvocationContext;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.ContentAssistantFacade;
import org.eclipse.jface.text.source.IAnnotationHover;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.ISourceViewerExtension;
import org.eclipse.jface.text.source.ISourceViewerExtension2;
import org.eclipse.jface.text.source.ISourceViewerExtension3;
import org.eclipse.jface.text.source.ISourceViewerExtension4;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.viewers.IInputProvider;
import org.eclipse.jface.viewers.IInputSelectionProvider;
import org.eclipse.jface.viewers.IPostSelectionProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.custom.StyledTextPrintOptions;
import org.eclipse.swt.custom.VerifyKeyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.texteditor.IFindReplaceTargetExtension2;

public class CompositeSourceViewer implements ISourceViewer,
		ISourceViewerExtension, ISourceViewerExtension2,
		ISourceViewerExtension3, ISourceViewerExtension4,
		IInputSelectionProvider, IEditingSupportRegistry,
		IPostSelectionProvider, ITextOperationTarget, 
		ITextOperationTargetExtension, ITextViewer,
		ITextViewerExtension, ITextViewerExtension2,
		ITextViewerExtension4,
		ITextViewerExtension5, ITextViewerExtension6,
		ITextViewerExtension7, ITextViewerExtension8,
		IWidgetTokenOwner, IWidgetTokenOwnerExtension {
	
	class CompositeFindReplaceTarget implements IFindReplaceTarget, IFindReplaceTargetExtension2, IFindReplaceTargetExtension3 {

		public boolean canPerformFind() {
			return getActiveSourceViewer().getFindReplaceTarget().canPerformFind();
		}

		public int findAndSelect(int widgetOffset, String findString,
				boolean searchForward, boolean caseSensitive, boolean wholeWord) {
			return getActiveSourceViewer().getFindReplaceTarget().findAndSelect(widgetOffset, findString, searchForward, caseSensitive, wholeWord);
		}

		public Point getSelection() {
			return getActiveSourceViewer().getFindReplaceTarget().getSelection();
		}

		public String getSelectionText() {
			return getActiveSourceViewer().getFindReplaceTarget().getSelectionText();
		}

		public boolean isEditable() {
			return getActiveSourceViewer().getFindReplaceTarget().isEditable();
		}

		public void replaceSelection(String text) {
			getActiveSourceViewer().getFindReplaceTarget().replaceSelection(text);
		}

		public boolean validateTargetState() {
			final IFindReplaceTarget findReplaceTarget = getActiveSourceViewer().getFindReplaceTarget();
			if (findReplaceTarget instanceof IFindReplaceTargetExtension2) {
				return ((IFindReplaceTargetExtension2) findReplaceTarget).validateTargetState();
			}
			return false;
		}

		public int findAndSelect(int offset, String findString,
				boolean searchForward, boolean caseSensitive,
				boolean wholeWord, boolean regExSearch) {
			final IFindReplaceTarget findReplaceTarget = getActiveSourceViewer().getFindReplaceTarget();
			if (findReplaceTarget instanceof IFindReplaceTargetExtension3) {
				return ((IFindReplaceTargetExtension3) findReplaceTarget).findAndSelect(offset, findString, searchForward, caseSensitive, wholeWord, regExSearch);
			}
			return -1;
		}

		public void replaceSelection(String text, boolean regExReplace) {
			final IFindReplaceTarget findReplaceTarget = getActiveSourceViewer().getFindReplaceTarget();
			if (findReplaceTarget instanceof IFindReplaceTargetExtension3) {
				((IFindReplaceTargetExtension3) findReplaceTarget).replaceSelection(text, regExReplace);
			}
		}
		
	}
	
	private ISourceViewer[] children;
	
	private ISourceViewer activeSourceViewer;
	
	private IFindReplaceTarget findReplaceTarget;
	
	public CompositeSourceViewer(int capacity) {
		children = new ISourceViewer[capacity];
	}
	
	private int firstChild() {
		return nextChild(-1);
	}
	
	private int nextChild(int i) {
		for (int j = i + 1; j < children.length; j++) {
			if (children[j] != null) {
				return j;
			}
		}
		return -1;
	}
	
	public ISourceViewer getSourceViewer(int index) {
		return children[index];
	}

	public void setSourceViewer(int index, ISourceViewer viewer) {
		children[index] = viewer;
	}
	
	public void removeSourceViewer(int index) {
		ISourceViewer oldActive = children[index];
		children[index] = null;
		if (activeSourceViewer == oldActive) {
			int first = firstChild();
			if (first != -1) {
				activeSourceViewer = children[first];
			} else {
				activeSourceViewer = null;
			}
		}
	}
	
	public ISourceViewer getActiveSourceViewer() {
		return activeSourceViewer;
	}
	
	public void setActiveSourceViewer(ISourceViewer viewer) {
		this.activeSourceViewer = viewer;
	}

	public void configure(SourceViewerConfiguration configuration) {
		getActiveSourceViewer().configure(configuration);
	}

	public IAnnotationModel getAnnotationModel() {
		return getActiveSourceViewer().getAnnotationModel();
	}

	public IRegion getRangeIndication() {
		return getActiveSourceViewer().getRangeIndication();
	}

	public void removeRangeIndication() {
		getActiveSourceViewer().removeRangeIndication();
	}

	public void setAnnotationHover(IAnnotationHover annotationHover) {
		getActiveSourceViewer().setAnnotationHover(annotationHover);
	}

	public void setDocument(IDocument document, IAnnotationModel annotationModel) {
		getActiveSourceViewer().setDocument(document, annotationModel);
	}

	public void setDocument(IDocument document,
			IAnnotationModel annotationModel, int modelRangeOffset,
			int modelRangeLength) {
		getActiveSourceViewer().setDocument(document, annotationModel, modelRangeOffset, modelRangeLength);
	}

	public void setRangeIndication(int offset, int length, boolean moveCursor) {
		getActiveSourceViewer().setRangeIndication(offset, length, moveCursor);
	}

	public void setRangeIndicator(Annotation rangeIndicator) {
		getActiveSourceViewer().setRangeIndicator(rangeIndicator);
	}

	public void showAnnotations(boolean show) {
		getActiveSourceViewer().showAnnotations(show);
	}

	public void activatePlugins() {
		getActiveSourceViewer().activatePlugins();
	}

	public void addTextInputListener(ITextInputListener listener) {
		getActiveSourceViewer().addTextInputListener(listener);
	}

	public void addTextListener(ITextListener listener) {
		getActiveSourceViewer().addTextListener(listener);
	}

	public void addViewportListener(IViewportListener listener) {
		getActiveSourceViewer().addViewportListener(listener);
	}

	public void changeTextPresentation(TextPresentation presentation,
			boolean controlRedraw) {
		getActiveSourceViewer().changeTextPresentation(presentation, controlRedraw);
	}

	public int getBottomIndex() {
		return getActiveSourceViewer().getBottomIndex();
	}

	public int getBottomIndexEndOffset() {
		return getActiveSourceViewer().getBottomIndexEndOffset();
	}

	public IDocument getDocument() {
		return getActiveSourceViewer().getDocument();
	}

	public IFindReplaceTarget getFindReplaceTarget() {
		if (findReplaceTarget == null) {
			findReplaceTarget = new CompositeFindReplaceTarget();
		}
		return findReplaceTarget;
	}

	public Point getSelectedRange() {
		return getActiveSourceViewer().getSelectedRange();
	}

	public ISelectionProvider getSelectionProvider() {
		// TODO A more explicit delegating strategy might be needed that does not assume that the underlying children implement ISelectionProvider themselves. 
		return this;
	}

	public ITextOperationTarget getTextOperationTarget() {
		// TODO A more explicit delegating strategy might be needed that does not assume that the underlying children implement ISelectionProvider themselves. 
		return this;
	}

	public StyledText getTextWidget() {
		return getActiveSourceViewer().getTextWidget();
	}

	public int getTopIndex() {
		return getActiveSourceViewer().getTopIndex();
	}

	public int getTopIndexStartOffset() {
		return getActiveSourceViewer().getTopIndexStartOffset();
	}

	public int getTopInset() {
		return getActiveSourceViewer().getTopInset();
	}

	public IRegion getVisibleRegion() {
		return getActiveSourceViewer().getVisibleRegion();
	}

	public void invalidateTextPresentation() {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			getSourceViewer(i).invalidateTextPresentation();
		}
	}

	public boolean isEditable() {
		return getActiveSourceViewer().isEditable();
	}

	public boolean overlapsWithVisibleRegion(int offset, int length) {
		return getActiveSourceViewer().overlapsWithVisibleRegion(offset, length);
	}

	public void removeTextInputListener(ITextInputListener listener) {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			getSourceViewer(i).removeTextInputListener(listener);
		}
	}

	public void removeTextListener(ITextListener listener) {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			getSourceViewer(i).removeTextListener(listener);
		}
	}

	public void removeViewportListener(IViewportListener listener) {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			getSourceViewer(i).removeViewportListener(listener);
		}
	}

	public void resetPlugins() {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			getSourceViewer(i).resetPlugins();
		}
	}

	public void resetVisibleRegion() {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			getSourceViewer(i).resetVisibleRegion();
		}
	}

	public void revealRange(int offset, int length) {
		getActiveSourceViewer().revealRange(offset, length);
	}

	public void setAutoIndentStrategy(IAutoIndentStrategy strategy,
			String contentType) {
		getActiveSourceViewer().setAutoIndentStrategy(strategy, contentType);
	}

	public void setDefaultPrefixes(String[] defaultPrefixes, String contentType) {
		getActiveSourceViewer().setDefaultPrefixes(defaultPrefixes, contentType);
	}

	public void setDocument(IDocument document) {
		getActiveSourceViewer().setDocument(document);
	}

	public void setDocument(IDocument document, int modelRangeOffset,
			int modelRangeLength) {
		getActiveSourceViewer().setDocument(document, modelRangeOffset, modelRangeLength);
	}

	public void setEditable(boolean editable) {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			getSourceViewer(i).setEditable(editable);
		}
	}

	public void setEventConsumer(IEventConsumer consumer) {
		getActiveSourceViewer().setEventConsumer(consumer);
	}

	public void setIndentPrefixes(String[] indentPrefixes, String contentType) {
		getActiveSourceViewer().setIndentPrefixes(indentPrefixes, contentType);
	}

	public void setSelectedRange(int offset, int length) {
		getActiveSourceViewer().setSelectedRange(offset, length);
	}

	public void setTextColor(Color color) {
		getActiveSourceViewer().setTextColor(color);
	}

	public void setTextColor(Color color, int offset, int length,
			boolean controlRedraw) {
		getActiveSourceViewer().setTextColor(color, offset, length, controlRedraw);
	}

	public void setTextDoubleClickStrategy(ITextDoubleClickStrategy strategy,
			String contentType) {
		getActiveSourceViewer().setTextDoubleClickStrategy(strategy, contentType);
	}

	public void setTextHover(ITextHover textViewerHover, String contentType) {
		getActiveSourceViewer().setTextHover(textViewerHover, contentType);
	}

	public void setTopIndex(int index) {
		getActiveSourceViewer().setTopIndex(index);
	}

	public void setUndoManager(IUndoManager undoManager) {
		getActiveSourceViewer().setUndoManager(undoManager);
	}

	public void setVisibleRegion(int offset, int length) {
		getActiveSourceViewer().setVisibleRegion(offset, length);
	}

	public void showAnnotationsOverview(boolean show) {
		for (int i = firstChild(); i != -1; i = nextChild(i)) {
			if (getSourceViewer(i) instanceof ISourceViewerExtension) {
				((ISourceViewerExtension) getSourceViewer(i)).showAnnotationsOverview(show);
			}
		}
	}

	public IAnnotationModel getVisualAnnotationModel() {
		if (getActiveSourceViewer() instanceof ISourceViewerExtension2) {
			return ((ISourceViewerExtension2) getActiveSourceViewer()).getVisualAnnotationModel();
		}
		return null;
	}

	public void unconfigure() {
		if (getActiveSourceViewer() instanceof ISourceViewerExtension2) {
			((ISourceViewerExtension2) getActiveSourceViewer()).unconfigure();
		}
	}

	public IAnnotationHover getCurrentAnnotationHover() {
		if (getActiveSourceViewer() instanceof ISourceViewerExtension3) {
			return ((ISourceViewerExtension3) getActiveSourceViewer()).getCurrentAnnotationHover();
		}
		return null;
	}

	public IQuickAssistAssistant getQuickAssistAssistant() {
		if (getActiveSourceViewer() instanceof ISourceViewerExtension3) {
			return ((ISourceViewerExtension3) getActiveSourceViewer()).getQuickAssistAssistant();
		}
		return null;
	}

	public IQuickAssistInvocationContext getQuickAssistInvocationContext() {
		if (getActiveSourceViewer() instanceof ISourceViewerExtension3) {
			return ((ISourceViewerExtension3) getActiveSourceViewer()).getQuickAssistInvocationContext();
		}
		return null;
	}

	public ContentAssistantFacade getContentAssistantFacade() {
		if (getActiveSourceViewer() instanceof ISourceViewerExtension4) {
			return ((ISourceViewerExtension4) getActiveSourceViewer()).getContentAssistantFacade();
		}
		return null;
	}

	public Object getInput() {
		if (getActiveSourceViewer() instanceof IInputProvider) {
			return ((IInputProvider) getActiveSourceViewer()).getInput();
		}
		return null;
	}

	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		if (getActiveSourceViewer() instanceof ISelectionProvider) {
			((ISelectionProvider) getActiveSourceViewer()).addSelectionChangedListener(listener);
		}
	}

	public ISelection getSelection() {
		if (getActiveSourceViewer() instanceof ISelectionProvider) {
			return ((ISelectionProvider) getActiveSourceViewer()).getSelection();
		}
		return null;
	}

	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
		if (getActiveSourceViewer() instanceof ISelectionProvider) {
			((ISelectionProvider) getActiveSourceViewer()).removeSelectionChangedListener(listener);
		}
	}

	public void setSelection(ISelection selection) {
		if (getActiveSourceViewer() instanceof ISelectionProvider) {
			((ISelectionProvider) getActiveSourceViewer()).setSelection(selection);
		}
	}

	public IEditingSupport[] getRegisteredSupports() {
		if (getActiveSourceViewer() instanceof IEditingSupportRegistry) {
			return ((IEditingSupportRegistry) getActiveSourceViewer()).getRegisteredSupports();
		}
		return null;
	}

	public void register(IEditingSupport support) {
		if (getActiveSourceViewer() instanceof IEditingSupportRegistry) {
			((IEditingSupportRegistry) getActiveSourceViewer()).register(support);
		}
	}

	public void unregister(IEditingSupport support) {
		if (getActiveSourceViewer() instanceof IEditingSupportRegistry) {
			((IEditingSupportRegistry) getActiveSourceViewer()).unregister(support);
		}
	}

	public void addPostSelectionChangedListener(ISelectionChangedListener listener) {
		if (getActiveSourceViewer() instanceof IPostSelectionProvider) {
			((IPostSelectionProvider) getActiveSourceViewer()).addPostSelectionChangedListener(listener);
		}
	}

	public void removePostSelectionChangedListener(
			ISelectionChangedListener listener) {
		if (getActiveSourceViewer() instanceof IPostSelectionProvider) {
			((IPostSelectionProvider) getActiveSourceViewer()).removePostSelectionChangedListener(listener);
		}
	}

	public boolean canDoOperation(int operation) {
		if (getActiveSourceViewer() instanceof ITextOperationTarget) {
			return ((ITextOperationTarget) getActiveSourceViewer()).canDoOperation(operation);
		}
		return false;
	}

	public void doOperation(int operation) {
		if (getActiveSourceViewer() instanceof ITextOperationTarget) {
			((ITextOperationTarget) getActiveSourceViewer()).doOperation(operation);
		}
	}

	public void enableOperation(int operation, boolean enable) {
		if (getActiveSourceViewer() instanceof ITextOperationTargetExtension) {
			((ITextOperationTargetExtension) getActiveSourceViewer()).enableOperation(operation, enable);
		}
	}

	public void appendVerifyKeyListener(VerifyKeyListener listener) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			((ITextViewerExtension) getActiveSourceViewer()).appendVerifyKeyListener(listener);
		}
	}

	public Control getControl() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			return ((ITextViewerExtension) getActiveSourceViewer()).getControl();
		}
		return null;
	}

	public int getMark() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			return ((ITextViewerExtension) getActiveSourceViewer()).getMark();
		}
		return 0;
	}

	public IRewriteTarget getRewriteTarget() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			return ((ITextViewerExtension) getActiveSourceViewer()).getRewriteTarget();
		}
		return null;
	}

	public void prependVerifyKeyListener(VerifyKeyListener listener) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			((ITextViewerExtension) getActiveSourceViewer()).prependVerifyKeyListener(listener);
		}
	}

	public void removeVerifyKeyListener(VerifyKeyListener listener) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			((ITextViewerExtension) getActiveSourceViewer()).removeVerifyKeyListener(listener);
		}
	}

	public void setMark(int offset) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			((ITextViewerExtension) getActiveSourceViewer()).setMark(offset);
		}
	}

	public void setRedraw(boolean redraw) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension) {
			((ITextViewerExtension) getActiveSourceViewer()).setRedraw(redraw);
		}
	}

	public void addPainter(IPainter painter) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			((ITextViewerExtension2) getActiveSourceViewer()).addPainter(painter);
		}
	}

	public ITextHover getCurrentTextHover() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			return ((ITextViewerExtension2) getActiveSourceViewer()).getCurrentTextHover();
		}
		return null;
	}

	public Point getHoverEventLocation() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			return ((ITextViewerExtension2) getActiveSourceViewer()).getHoverEventLocation();
		}
		return null;
	}

	public void invalidateTextPresentation(int offset, int length) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			((ITextViewerExtension2) getActiveSourceViewer()).invalidateTextPresentation(offset, length);
		}
	}

	public void prependAutoEditStrategy(IAutoEditStrategy strategy,
			String contentType) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			((ITextViewerExtension2) getActiveSourceViewer()).prependAutoEditStrategy(strategy, contentType);
		}
	}

	public void removeAutoEditStrategy(IAutoEditStrategy strategy,
			String contentType) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			((ITextViewerExtension2) getActiveSourceViewer()).removeAutoEditStrategy(strategy, contentType);
		}
	}

	public void removePainter(IPainter painter) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			((ITextViewerExtension2) getActiveSourceViewer()).removePainter(painter);
		}
	}

	public void removeTextHovers(String contentType) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			((ITextViewerExtension2) getActiveSourceViewer()).removeTextHovers(contentType);
		}
	}

	public void setTextHover(ITextHover textViewerHover, String contentType,
			int stateMask) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension2) {
			((ITextViewerExtension2) getActiveSourceViewer()).setTextHover(textViewerHover, contentType, stateMask);
		}
	}

	public void addTextPresentationListener(ITextPresentationListener listener) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension4) {
			((ITextViewerExtension4) getActiveSourceViewer()).addTextPresentationListener(listener);
		}
	}

	public boolean moveFocusToWidgetToken() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension4) {
			return ((ITextViewerExtension4) getActiveSourceViewer()).moveFocusToWidgetToken();
		}
		return false;
	}

	public void removeTextPresentationListener(ITextPresentationListener listener) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension4) {
			((ITextViewerExtension4) getActiveSourceViewer()).removeTextPresentationListener(listener);
		}
	}

	public IUndoManager getUndoManager() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension6) {
			return ((ITextViewerExtension6) getActiveSourceViewer()).getUndoManager();
		}
		return null;
	}

	public void setHyperlinkDetectors(IHyperlinkDetector[] hyperlinkDetectors,
			int eventStateMask) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension6) {
			((ITextViewerExtension6) getActiveSourceViewer()).setHyperlinkDetectors(hyperlinkDetectors, eventStateMask);
		}
	}

	public void setTabsToSpacesConverter(IAutoEditStrategy converter) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension7) {
			((ITextViewerExtension7) getActiveSourceViewer()).setTabsToSpacesConverter(converter);
		}
	}

	public void print(StyledTextPrintOptions options) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension8) {
			((ITextViewerExtension8) getActiveSourceViewer()).print(options);
		}
	}

	public void setHoverEnrichMode(EnrichMode mode) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension8) {
			((ITextViewerExtension8) getActiveSourceViewer()).setHoverEnrichMode(mode);
		}
	}

	public void releaseWidgetToken(IWidgetTokenKeeper tokenKeeper) {
		if (getActiveSourceViewer() instanceof IWidgetTokenOwner) {
			((IWidgetTokenOwner) getActiveSourceViewer()).releaseWidgetToken(tokenKeeper);
		}
	}

	public boolean requestWidgetToken(IWidgetTokenKeeper requester) {
		if (getActiveSourceViewer() instanceof IWidgetTokenOwner) {
			return ((IWidgetTokenOwner) getActiveSourceViewer()).requestWidgetToken(requester);
		}
		return false;
	}

	public boolean requestWidgetToken(IWidgetTokenKeeper requester, int priority) {
		if (getActiveSourceViewer() instanceof IWidgetTokenOwnerExtension) {
			return ((IWidgetTokenOwnerExtension) getActiveSourceViewer()).requestWidgetToken(requester, priority);
		}
		return false;
	}

	public IRegion getModelCoverage() {
		if (getActiveSourceViewer() instanceof ITextViewerExtension3) {
			return ((ITextViewerExtension3) getActiveSourceViewer()).getModelCoverage();
		}
		return null;
	}

	public int modelLine2WidgetLine(int modelLine) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).modelLine2WidgetLine(modelLine);
		}
		return modelLine;
	}

	public int modelOffset2WidgetOffset(int modelOffset) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).modelOffset2WidgetOffset(modelOffset);
		}
		return modelOffset;
	}

	public IRegion modelRange2WidgetRange(IRegion modelRange) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).modelRange2WidgetRange(modelRange);
		}
		return modelRange;
	}

	public int widgetLineOfWidgetOffset(int widgetOffset) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).widgetLineOfWidgetOffset(widgetOffset);
		}
		return 0;
	}

	public int widgetOffset2ModelOffset(int widgetOffset) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).widgetOffset2ModelOffset(widgetOffset);
		}
		return widgetOffset;
	}

	public IRegion widgetRange2ModelRange(IRegion widgetRange) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).widgetRange2ModelRange(widgetRange); 
		}
		return widgetRange;
	}

	public int widgetlLine2ModelLine(int widgetLine) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).widgetLine2ModelLine(widgetLine);
		}
		return widgetLine;
	}

	public boolean exposeModelRange(IRegion modelRange) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).exposeModelRange(modelRange);
		}
		return false;
	}

	public IRegion[] getCoveredModelRanges(IRegion modelRange) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).getCoveredModelRanges(modelRange);
		}
		return null;
	}

	public int widgetLine2ModelLine(int widgetLine) {
		if (getActiveSourceViewer() instanceof ITextViewerExtension5) {
			return ((ITextViewerExtension5) getActiveSourceViewer()).widgetLine2ModelLine(widgetLine);
		}
		return widgetLine;
	}

}
