package org.eclipse.spliteditor;

import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.spliteditor.tests.SplittableContainer;
import org.eclipse.spliteditor.tests.SplittableContainerClient;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Sash;
import org.eclipse.ui.editors.text.TextEditor;

public class SplitTextEditor extends TextEditor {
	
	private static final int E_MASTER = 0;
	
	private static final int E_SLAVE = 1;

	private SplittableContainer splitContainer;
	
	private CompositeSourceViewer sourceViewer;
	
	private int creatingChild;
	
	public void createPartControl(Composite parent) {
		splitContainer = new SplittableContainer(parent, new SplittableContainerClient() {
		
			public Control createSlave(SplittableContainer container) {
				// TODO store master/slave decoration support objects and switch between them and dispose appropriately
				fSourceViewerDecorationSupport = null;
				ISourceViewer activeSourceViewer = sourceViewer.getActiveSourceViewer();
				creatingChild = E_SLAVE;
				SplitTextEditor.super.createPartControl(container);
				creatingChild = -1;
				sourceViewer.setActiveSourceViewer(activeSourceViewer);
				handleInsertModeChanged();
				Control[] ch = container.getChildren();
				for (int i = 0; i < ch.length; i++) {
					if (ch[i] != container.getMaster() && !(ch[i] instanceof Sash)) {
						return ch[i];
					}
				}
				return null;
			}

			public Control createMaster(SplittableContainer container) {
				creatingChild = E_MASTER;
				SplitTextEditor.super.createPartControl(container);
				creatingChild = -1;
				Control[] ch = container.getChildren();
				return ch[0];
			}

			public void slaveDismissed(SplittableContainer container) {
				sourceViewer.removeSourceViewer(E_SLAVE);
				setFocus();
				handleInsertModeChanged();
			}
		}, SWT.NONE);
	}

	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		if (sourceViewer == null) {
			sourceViewer = new CompositeSourceViewer(2);
		}
		final ISourceViewer childSourceViewer = super.createSourceViewer(parent, ruler, styles);
		sourceViewer.setSourceViewer(creatingChild, childSourceViewer);
		sourceViewer.setActiveSourceViewer(childSourceViewer);
		parent.addListener(SWT.Activate, new Listener() {
			public void handleEvent(Event event) {
				if (event.type == SWT.Activate) {
					sourceViewer.setActiveSourceViewer(childSourceViewer);
					handleInsertModeChanged();
				}
			}
		});
		return sourceViewer;
	}

	protected void createUndoRedoActions() {
		// Avoid recreating undo/redo actions on split. It's OK, only one set of
		// these actions is needed, because these actions are bound to the
		// underlying document, not the source viewer.
		if (creatingChild == E_MASTER) {
			super.createUndoRedoActions();
		}
	}
	
}
