/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.spliteditor.tests;
 
/*
 * Sash example snippet: create a sash (allow it to be moved)
 *
 * For a list of all SWT example snippets see
 * http://www.eclipse.org/swt/snippets/
 */
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Sash;
import org.eclipse.swt.widgets.Shell;

public class Snippet54 {
	
	static class SashData {
		StyledText master;
		StyledText slave;

		int position, size;
	}

	public static void main (String [] args) {
		Display display = new Display ();
		Shell shell = new Shell (display);
		
		shell.setLayout(new FillLayout());
	
		final Composite container = new Composite(shell, SWT.NONE);
	
		final SashData data = new SashData();
		final Sash sash = new Sash (container, SWT.HORIZONTAL);
		sash.setSize(5, 5);
		sash.addListener (SWT.Selection, new Listener () {
			public void handleEvent (Event e) {
				if ((e.detail & SWT.DRAG) == 0) {
					data.position = e.y;
					data.size = container.getSize().y;
					
					if (data.position < 10) {
						data.position = 0;
						if (data.slave != null) {
							data.slave.dispose();
							data.slave = null;
						}
					} else {
						if (data.slave == null) {
							data.slave = new StyledText(container, SWT.NONE);
							data.slave.setText(data.master.getText());
						}
					}
					
					container.layout();
				}
			}
		});
		sash.setLayoutData(data);
	
		data.master = new StyledText(container, SWT.NONE);
		
		container.setLayout(new Layout() {

			protected void layout(Composite composite, boolean flushCache) {
				Rectangle area = composite.getClientArea();
				if (area.width <= 1 || area.height <= 1) return;
				
				if (data.size == 0) {
					data.size = area.height;
				}
				
				int sashPos = (data.position * area.height) / data.size;
				
				if (data.slave != null) {
					data.slave.setBounds(area.x, area.y, area.width, sashPos);
				}
				
				int sashSize = sash.getSize().y;
				sash.setBounds(area.x, area.y + sashPos, area.width, sashSize);
				int sashBottom = sashPos + sashSize;
				
				data.master.setBounds(area.x, area.y + sashBottom, area.width, area.height - sashBottom);
			}

			protected Point computeSize(Composite composite, int hint, int hint2,
					boolean flushCache) {
				return null;
			}
		});
		
		shell.open ();
		while (!shell.isDisposed ()) {
			if (!display.readAndDispatch ()) display.sleep ();
		}
		display.dispose ();
	}
} 
